package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.space.service.ISpaceYesterdayGreenRateCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 昨日绿电占比Cloud模块
 *
 * @author ZWT
 * @date 2024/11/27 15:17
 */
@XService
public class SpaceYesterdayGreenRateCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceYesterdayGreenRateCloudService {

    /**
     * 昨日绿电占比
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult yesterdayGreenRateJob(XContext context) {
        //取昨日
        DateTime beginOfYesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //取昨日间开优化井
        SpaceOptimizeShortDurationViewMapper spaceMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationViewList = spaceMapper.selectResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(beginOfYesterday)
                .build());
        Map<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> durationCollect;
        Map<String, Map<Date, BigDecimal>> activeMap;
        Map<String, Map<Date, BigDecimal>> predictionMap;
        if (CollUtil.isEmpty(durationViewList)) {
            durationCollect = Collections.emptyMap();
            activeMap = Collections.emptyMap();
            predictionMap = Collections.emptyMap();
        } else {
            //间开区间Map
            durationCollect = durationViewList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName,
                                    Collectors.groupingBy(SpaceOptimizeShortDurationView::getStartSeq,
                                            Collectors.groupingBy(SpaceOptimizeShortDurationView::getIsOptimize))
                            )
                    );
            //取昨日实际发电功率
            IThirdPowerCloudService cloudService = context.getBean(IThirdPowerCloudService.class);
            XListResult<GetThirdActivePowerOutput> result = cloudService.queryPhotovoltaicPower30(context, GetThirdActivePowerInput.builder()
                    .stationNames(durationCollect.keySet())
                    .createDate(beginOfYesterday)
                    .build());
            result.throwIfFail();
            List<GetThirdActivePowerOutput> powerList = result.getResult();
            if (CollUtil.isEmpty(powerList)) {
                activeMap = Collections.emptyMap();
            } else {
                activeMap = powerList.stream()
                        .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName,
                                Collectors.toMap(GetThirdActivePowerOutput::getCreateDate, GetThirdActivePowerOutput::getPhotovoltaicPower)));
            }
            //昨日预测发电功率
            IPlantPredictedPowerCloudService service = context.getBean(IPlantPredictedPowerCloudService.class);
            XListResult<GetPredictedPowerViewOutput> predictionResult = service.queryPredictionPower30(context, GetPredictedPowerViewInput.builder()
                    .stationNames(durationCollect.keySet())
                    .dataDate(beginOfYesterday)
                    .build());
            predictionResult.throwIfFail();
            List<GetPredictedPowerViewOutput> predictionList = predictionResult.getResult();
            if (CollUtil.isEmpty(predictionList)) {
                predictionMap = Collections.emptyMap();
            } else {
                predictionMap = predictionList.stream()
                        .collect(Collectors.groupingBy(GetPredictedPowerViewOutput::getStationName,
                                Collectors.toMap(GetPredictedPowerViewOutput::getCreateTime, GetPredictedPowerViewOutput::getPredictPower)));
            }
        }
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        String lineName;
        Map<Date, BigDecimal> activePowerMap;
        Map<Date, BigDecimal> predictionPowerMap;
        List<SpaceOptimizeShortDurationView> unOptimizeList;
        List<SpaceOptimizeShortDurationView> optimizeList;
        TreeMap<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> wellSortMap;
        for (Map.Entry<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> lineEntry : durationCollect.entrySet()) {
            lineName = lineEntry.getKey();
            if (activeMap.containsKey(lineName)) {
                activePowerMap = activeMap.get(lineName);
            } else {
                activePowerMap = Collections.emptyMap();
            }
            if (predictionMap.containsKey(lineName)) {
                predictionPowerMap = predictionMap.get(lineName);
            } else {
                predictionPowerMap = Collections.emptyMap();
            }
            //按顺序排
            wellSortMap = new TreeMap<>(lineEntry.getValue());
            BigDecimal serviceRating = BigDecimal.ZERO;
            for (Map.Entry<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> sortEntry : wellSortMap.entrySet()) {
                //未优化间开
                unOptimizeList = sortEntry.getValue().get(1);
                if (CollUtil.isEmpty(unOptimizeList)) {
                    continue;
                }
                //优化后间开
                optimizeList = sortEntry.getValue().get(0);
                //累加运行功率
                serviceRating = serviceRating.add(ObjectUtil.defaultIfNull(unOptimizeList.get(0).getServiceRating(), BigDecimal.ZERO));
                //计算优化前后功率
                for (DateTime dateTime : rangeToList) {
                    BigDecimal activePower;
                    if (activePowerMap.containsKey(dateTime)) {
                        activePower = activePowerMap.get(dateTime);
                    }
                    BigDecimal predictionPower;
                    if (predictionPowerMap.containsKey(dateTime)) {
                        predictionPower = predictionPowerMap.get(dateTime);
                    }
                }
            }
        }
        return XServiceResult.OK;
    }
}